/* function pairwise1.cpp
 * Compile:
 * mex pairwise1.cpp
 */

/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "mex.h"
#include <string.h>
#include <math.h>

inline int index2(int r, int c, int R) {
    return c*R + r;
}

inline int index3(int r, int c, int t, int R, int C) {
    return t*(R*C) + c*R + r;
}

void mexFunction(int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[])
{

  const int * sizeCurr = mxGetDimensions(prhs[4]);
  const int R = sizeCurr[0];
  const int currC = sizeCurr[1];
  const int * sizeInput = mxGetDimensions(prhs[6]);
  const int C = sizeInput[1];

  //get all input pointers
  char * domain = mxArrayToString(prhs[0]);
  double * term = mxGetPr(prhs[1]);
  double * pairwiseorder = mxGetPr(prhs[2]);
  double * blend = mxGetPr(prhs[3]);
  double * imCurr = mxGetPr(prhs[4]);
  double * imMagnitude = mxGetPr(prhs[5]);
  double * imInput = mxGetPr(prhs[6]);
  double * imInputMagnitude = mxGetPr(prhs[7]);
  double * imShift = mxGetPr(prhs[8]);
  double * imNumber = mxGetPr(prhs[9]);
  int r1 = (int)*mxGetPr(prhs[10]);
  int c1 = (int)*mxGetPr(prhs[11]);
  int r2 = (int)*mxGetPr(prhs[12]);
  int c2 = (int)*mxGetPr(prhs[13]);
  int r3 = 0;
  int c3 = 0;
  int r4 = 0;
  int c4 = 0;
  if(nrhs>14) {
    r3 = (int)*mxGetPr(prhs[14]);
    c3 = (int)*mxGetPr(prhs[15]);
    r4 = (int)*mxGetPr(prhs[16]);
    c4 = (int)*mxGetPr(prhs[17]);
  }

  mwSize sizeOut[1];
  sizeOut[0] = 1;
  
  plhs[0] = mxCreateNumericArray(1,sizeOut,mxDOUBLE_CLASS,mxREAL);
  double * energy = mxGetPr(plhs[0]);

  if(strcmp(domain,"curr")==0) {
    // Curr
    if((*term)==1) {
      *energy = pow( fabs( imCurr[index3(r1-1,c1-1,0,R,currC)] - imCurr[index3(r2-1,c2-1,0,R,currC)]),*pairwiseorder ) +
	        pow( fabs( imCurr[index3(r1-1,c1-1,1,R,currC)] - imCurr[index3(r2-1,c2-1,1,R,currC)]),*pairwiseorder ) +
	        pow( fabs( imCurr[index3(r1-1,c1-1,2,R,currC)] - imCurr[index3(r2-1,c2-1,2,R,currC)]),*pairwiseorder );
    } else {
      *energy = pow( fabs( (1-*blend)*(imCurr[index3(r1-1,c1-1,0,R,currC)] - imCurr[index3(r2-1,c2-1,0,R,currC)]) +
			   *blend*(imCurr[index3(r3-1,c3-1,0,R,currC)] - imCurr[index3(r4-1,c4-1,0,R,currC)])),*pairwiseorder ) +
	        pow( fabs( (1-*blend)*(imCurr[index3(r1-1,c1-1,1,R,currC)] - imCurr[index3(r2-1,c2-1,1,R,currC)]) + 
			   *blend*(imCurr[index3(r3-1,c3-1,1,R,currC)] - imCurr[index3(r4-1,c4-1,1,R,currC)])),*pairwiseorder ) +
        	pow( fabs( (1-*blend)*(imCurr[index3(r1-1,c1-1,2,R,currC)] - imCurr[index3(r2-1,c2-1,2,R,currC)]) +
			   *blend*(imCurr[index3(r3-1,c3-1,2,R,currC)] - imCurr[index3(r4-1,c4-1,2,R,currC)])),*pairwiseorder );
    }
  } else {
    // Input
    if((*term)==1) {
      *energy = pow( fabs( imInput[index3(r1-1,c1-1,0,R,C)] - imInput[index3(r2-1,c2-1,0,R,C)]),*pairwiseorder ) +
	        pow( fabs( imInput[index3(r1-1,c1-1,1,R,C)] - imInput[index3(r2-1,c2-1,1,R,C)]),*pairwiseorder ) +
         	pow( fabs( imInput[index3(r1-1,c1-1,2,R,C)] - imInput[index3(r2-1,c2-1,2,R,C)]),*pairwiseorder );
    } else if((*term)==2) {
      *energy = pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,0,R,currC)] - imInput[index3(r2-1,c2-1,0,R,C)]),*pairwiseorder ) +
        	pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,1,R,currC)] - imInput[index3(r2-1,c2-1,1,R,C)]),*pairwiseorder ) +
        	pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,2,R,currC)] - imInput[index3(r2-1,c2-1,2,R,C)]),*pairwiseorder );
    } else if((*term)==3) {
      *energy = pow( fabs( imInput[index3(r1-1,c1-1,0,R,C)] - imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,0,R,currC)]),*pairwiseorder ) +
        	pow( fabs( imInput[index3(r1-1,c1-1,1,R,C)] - imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,1,R,currC)]),*pairwiseorder ) +
          	pow( fabs( imInput[index3(r1-1,c1-1,2,R,C)] - imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,2,R,currC)]),*pairwiseorder );
    } else {
			    *energy = pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,0,R,currC)] - imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,0,R,currC)]),*pairwiseorder ) +
				      pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,1,R,currC)] - imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,1,R,currC)]),*pairwiseorder ) +
				      pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,2,R,currC)] - imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,2,R,currC)]),*pairwiseorder );
    }
  }
  
  mxFree(domain);
  return;
} 
