/* function pairwise2.cpp
 * Compile:
 * mex pairwise2.cpp
 */

/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "mex.h"
#include <string.h>
#include <math.h>
#include <matrix.h>

#include <assert.h>

inline int index2(int r, int c, int R) {
    return c*R + r;
}

inline int index3(int r, int c, int t, int R, int C) {
    return t*(R*C) + c*R + r;
}

inline int signum(int x) {
  return (x>0) - (x<0);
}

void mexFunction(int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[])
{

  const mwSize * sizeCurr = mxGetDimensions(prhs[4]);
  const int R = (int)sizeCurr[0];
  const int currC = (int)sizeCurr[1];
  const mwSize * sizeInput = mxGetDimensions(prhs[6]);
  const int C = (int)sizeInput[1];

#ifndef NDEBUG
  if(!((nrhs==14)||(nrhs==18)))
    mexErrMsgTxt("Needs 14 or 18 inputs!");
  if(!(nlhs==1))
    mexErrMsgTxt("Needs 1 output!");
#endif

  //get all input pointers
  assert(mxIsChar(prhs[0]));
  assert(mxIsDouble(prhs[1]));
  assert(mxIsDouble(prhs[2]));
  assert(mxIsDouble(prhs[3]));
  assert(mxIsDouble(prhs[4]));
  assert(mxIsDouble(prhs[5]));
  assert(mxIsDouble(prhs[6]));
  assert(mxIsDouble(prhs[7]));
  assert(mxIsDouble(prhs[8]));
  assert(mxIsDouble(prhs[9]));
  assert(mxIsDouble(prhs[10]));
  assert(mxIsDouble(prhs[11]));
  assert(mxIsDouble(prhs[12]));
  assert(mxIsDouble(prhs[13]));
  if(nrhs>14){
    assert(mxIsDouble(prhs[14]));
    assert(mxIsDouble(prhs[15]));
    assert(mxIsDouble(prhs[16]));
    assert(mxIsDouble(prhs[17]));
  }

  char * domain = mxArrayToString(prhs[0]);
#ifndef NDEBUG
  if(domain==NULL)
    mexErrMsgTxt("Unable to get character array for domain");
#endif
  double * term = (double*)mxGetData(prhs[1]);
  double * pairwiseorder = (double*)mxGetData(prhs[2]);
  double * blend = (double*)mxGetData(prhs[3]);
  double * imCurr = (double*)mxGetData(prhs[4]);
  double * imMagnitude = (double*)mxGetData(prhs[5]);
  double * imInput = (double*)mxGetData(prhs[6]);
  double * imInputMagnitude = (double*)mxGetData(prhs[7]);
  double * imShift = (double*)mxGetData(prhs[8]);
  double * imNumber = (double*)mxGetData(prhs[9]);
  int r1 = (int)*(double*)mxGetData(prhs[10]);
  int c1 = (int)*(double*)mxGetData(prhs[11]);
  int r2 = (int)*(double*)mxGetData(prhs[12]);
  int c2 = (int)*(double*)mxGetData(prhs[13]);
  int r3 = 0;
  int c3 = 0;
  int r4 = 0;
  int c4 = 0;
  if(nrhs>14) {
    r3 = (int)*(double*)mxGetData(prhs[14]);
    c3 = (int)*(double*)mxGetData(prhs[15]);
    r4 = (int)*(double*)mxGetData(prhs[16]);
    c4 = (int)*(double*)mxGetData(prhs[17]);
    }

  double energy = 0.0;

  if(strcmp(domain,"curr")==0) {
    // Curr
    if((*term)==1) {
      assert(r1>0);
      assert(r1<=R);
      assert(c1>0);
      assert(c1<=currC);

      assert(r2>0);
      assert(r2<=R);
      assert(c2>0);
      assert(c2<=currC);

      assert((c1+(int)imShift[index2(r2-1,c2-1,R)])<=currC);
      assert((c2+(int)imShift[index2(r1-1,c1-1,R)])<=currC);

      energy = pow( fabs( imCurr[index3(r1-1,c1-1,0,R,currC)] - imInput[index3(r1-1,c1+(int)imShift[index2(r2-1,c2-1,R)]-1,0,R,C)]),*pairwiseorder ) +
               pow( fabs( imCurr[index3(r1-1,c1-1,1,R,currC)] - imInput[index3(r1-1,c1+(int)imShift[index2(r2-1,c2-1,R)]-1,1,R,C)]),*pairwiseorder ) +
               pow( fabs( imCurr[index3(r1-1,c1-1,2,R,currC)] - imInput[index3(r1-1,c1+(int)imShift[index2(r2-1,c2-1,R)]-1,2,R,C)]),*pairwiseorder ) +

	       pow( fabs( imCurr[index3(r2-1,c2-1,0,R,currC)] - imInput[index3(r2-1,c2+(int)imShift[index2(r1-1,c1-1,R)]-1,0,R,C)]),*pairwiseorder ) +
	       pow( fabs( imCurr[index3(r2-1,c2-1,1,R,currC)] - imInput[index3(r2-1,c2+(int)imShift[index2(r1-1,c1-1,R)]-1,1,R,C)]),*pairwiseorder ) +
	       pow( fabs( imCurr[index3(r2-1,c2-1,2,R,currC)] - imInput[index3(r2-1,c2+(int)imShift[index2(r1-1,c1-1,R)]-1,2,R,C)]),*pairwiseorder );
    } else {

      int c_gap = 0;
      if((c2>c1)&&(r1==r2))
	c_gap = 1;

      assert(r1>0);
      assert(r1<=R);
      assert(c1>0);
      assert(c1<=currC);

      assert(r2>0);
      assert(r2<=R);
      assert(c2>0);
      assert(c2<=currC);

      assert(r3>0);
      assert(r3<=R);
      assert(c3>0);
      assert(c3<=currC);

      assert(r4>0);
      assert(r4<=R);
      assert(c4>0);
      assert(c4<=currC);

      assert((c2-c_gap+(int)imShift[index2(r2-1,c2-1,R)])<=C);
      assert((c1+c_gap+(int)imShift[index2(r1-1,c1-1,R)])<=C);
      
      energy = pow( fabs( (1-*blend)*imCurr[index3(r1-1,c1-1,0,R,currC)] + *blend*imCurr[index3(r3-1,c3-1,0,R,currC)] - imInput[index3(r1-1,c2-c_gap+(int)imShift[index2(r2-1,c2-1,R)]-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( (1-*blend)*imCurr[index3(r1-1,c1-1,1,R,currC)] + *blend*imCurr[index3(r3-1,c3-1,1,R,currC)] - imInput[index3(r1-1,c2-c_gap+(int)imShift[index2(r2-1,c2-1,R)]-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( (1-*blend)*imCurr[index3(r1-1,c1-1,2,R,currC)] + *blend*imCurr[index3(r3-1,c3-1,2,R,currC)] - imInput[index3(r1-1,c2-c_gap+(int)imShift[index2(r2-1,c2-1,R)]-1,2,R,C)]),*pairwiseorder ) +
	
	pow( fabs( (1-*blend)*imCurr[index3(r2-1,c2-1,0,R,currC)] + *blend*imCurr[index3(r4-1,c4-1,0,R,currC)] - imInput[index3(r2-1,c1+c_gap+(int)imShift[index2(r1-1,c1-1,R)]-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( (1-*blend)*imCurr[index3(r2-1,c2-1,1,R,currC)] + *blend*imCurr[index3(r4-1,c4-1,1,R,currC)] - imInput[index3(r2-1,c1+c_gap+(int)imShift[index2(r1-1,c1-1,R)]-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( (1-*blend)*imCurr[index3(r2-1,c2-1,2,R,currC)] + *blend*imCurr[index3(r4-1,c4-1,2,R,currC)] - imInput[index3(r2-1,c1+c_gap+(int)imShift[index2(r1-1,c1-1,R)]-1,2,R,C)]),*pairwiseorder );
    }
  } else {
    // Input
    if((*term)==1) {
      assert(r1>0);
      assert(r1<=R);
      assert(c1>0);
      assert(c1<=C);

      assert(r2>0);
      assert(r2<=R);
      assert(c2>0);
      assert(c2<=C);

      assert((c2+signum(c1-c2))<=C);
      assert((c1+signum(c2-c1))<=C);

      energy = pow( fabs( imInput[index3(r1-1,c1-1,0,R,C)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r1-1,c1-1,1,R,C)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r1-1,c1-1,2,R,C)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,2,R,C)]),*pairwiseorder ) +
			      
	pow( fabs( imInput[index3(r2-1,c2-1,0,R,C)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r2-1,c2-1,1,R,C)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r2-1,c2-1,2,R,C)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,2,R,C)]),*pairwiseorder );
    } else if((*term)==2) {
      assert(r1>0);
      assert(r1<=R);

      assert(r2>0);
      assert(r2<=R);
      assert(c2>0);
      assert(c2<=C);

      assert(((int)imNumber[index2(r1-1,c1-1,R)])<=currC);
      assert((c2+signum(c1-c2))<=C);
      assert((c1+signum(c2-c1))<=C);

      energy = pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,0,R,currC)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,1,R,currC)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,2,R,currC)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,2,R,C)]),*pairwiseorder ) +
			      
	pow( fabs( imInput[index3(r2-1,c2-1,0,R,C)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r2-1,c2-1,1,R,C)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r2-1,c2-1,2,R,C)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,2,R,C)]),*pairwiseorder );
    } else if((*term)==3) {
      assert(r1>0);
      assert(r1<=R);
      assert(c1>0);
      assert(c1<=C);

      assert(r2>0);
      assert(r2<=R);
      
      assert(((int)imNumber[index2(r2-1,c2-1,R)])<=currC);
      assert((c2+signum(c1-c2))<=C);
      assert((c1+signum(c2-c1))<=C);

      energy = pow( fabs( imInput[index3(r1-1,c1-1,0,R,C)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r1-1,c1-1,1,R,C)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imInput[index3(r1-1,c1-1,2,R,C)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,2,R,C)]),*pairwiseorder ) +

	pow( fabs( imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,0,R,currC)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,1,R,currC)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,2,R,currC)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,2,R,C)]),*pairwiseorder );
    } else {
      assert(r1>0);
      assert(r1<=R);

      assert(r2>0);
      assert(r2<=R);
      
      assert(((int)imNumber[index2(r1-1,c1-1,R)])<=currC);
      assert(((int)imNumber[index2(r2-1,c2-1,R)])<=currC);
      assert((c2+signum(c1-c2))<=C);
      assert((c1+signum(c2-c1))<=C);

      energy = pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,0,R,currC)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,1,R,currC)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r1-1,(int)imNumber[index2(r1-1,c1-1,R)]-1,2,R,currC)] - imInput[index3(r1-1,c2+signum(c1-c2)-1,2,R,C)]),*pairwiseorder ) +

	pow( fabs( imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,0,R,currC)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,0,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,1,R,currC)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,1,R,C)]),*pairwiseorder ) +
	pow( fabs( imCurr[index3(r2-1,(int)imNumber[index2(r2-1,c2-1,R)]-1,2,R,currC)] - imInput[index3(r2-1,c1+signum(c2-c1)-1,2,R,C)]),*pairwiseorder );
    }
    }

  plhs[0] = mxCreateDoubleScalar(energy);
#ifndef NDEBUG
  if(plhs[0]==NULL)
    mexErrMsgTxt("Cannot create output pointer");
#endif

  mxFree(domain);
  return;
} 
