#ifndef SEAMCARVE_H
#define SEAMCARVE_H

/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

// options

// prototypes

int index2(int r, int c, int R);
int index3(int r, int c, int t, int R, int C);
int signum(int x);

double pairwise0(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c);
double pairwise1(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c);
double pairwise2(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c);

class absPairwise {
 public:
  virtual double operator()(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c) const=0; 
};

class cntPairwise0 : public absPairwise {
 public:
  virtual double operator()(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c) const {
    return pairwise0(R, currC, C, domain, term, pairwiseorder, blend, imCurr, imMagnitude, imInput, imInputMagnitude, imShift, imNumber, r, c);
  }
};

class cntPairwise1 : public absPairwise {
 public:
  virtual double operator()(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c) const {
    return pairwise1(R, currC, C, domain, term, pairwiseorder, blend, imCurr, imMagnitude, imInput, imInputMagnitude, imShift, imNumber, r, c);
  }
};

class cntPairwise2 : public absPairwise {
 public:
  virtual double operator()(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c) const {
    return pairwise2(R, currC, C, domain, term, pairwiseorder, blend, imCurr, imMagnitude, imInput, imInputMagnitude, imShift, imNumber, r, c);
  }
};

double shift1(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c);
double shift2(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c);
double shift3(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c);
double shift4(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c);
double shift5(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c);

class absShift {
 public:
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const=0;
  virtual int null() const=0;
};

class cntShiftNULL : public absShift {
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const {
    return 1.0;
  }
  virtual int null() const {
    return 1;
  }
};

class cntShift1 : public absShift {
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const {
    return shift1(R, C, it, domain, term, shiftweight, imShift, imInputUnary, r, c);
  }
  virtual int null() const {
    return 0;
  }
};

class cntShift2 : public absShift {
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const {
    return shift2(R, C, it, domain, term, shiftweight, imShift, imInputUnary, r, c);
  }
  virtual int null() const {
    return 0;
  }
};

class cntShift3 : public absShift {
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const {
    return shift3(R, C, it, domain, term, shiftweight, imShift, imInputUnary, r, c);
  }
  virtual int null() const {
    return 0;
  }
};

class cntShift4 : public absShift {
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const {
    return shift4(R, C, it, domain, term, shiftweight, imShift, imInputUnary, r, c);
  }
  virtual int null() const {
    return 0;
  }
};

class cntShift5 : public absShift {
  virtual double operator()(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) const {
    return shift5(R, C, it, domain, term, shiftweight, imShift, imInputUnary, r, c);
  }
  virtual int null() const {
    return 0;
  }
};

class absOp {
 public:
  virtual double operator()(double x, double y) const=0;
  virtual int null() const=0;
};

class cntOpNULL : public absOp {
  virtual double operator()(double x, double y) const {
    return 1.0;
  }
  virtual int null() const {
    return 1;
  }
};

class cntOpPlus : public absOp {
  virtual double operator()(double x, double y) const {
    return x+y;
  }
  virtual int null() const {
    return 0;
  }
};

class cntOpTimes : public absOp {
  virtual double operator()(double x, double y) const {
    return x*y;
  }
  virtual int null() const {
    return 0;
  }
};

int sc_seamcarve(int const R, int const C, int const pairwise, double const pairwiseorder, int const unary, double const unaryorder, double const unaryweight, int const unaryinput, int const shiftterm, double const shiftweight, int const refinement, double const blend, int const total, int const logspeedup, double const infinity, double const un_small, int const S, absPairwise const * const fncPairwise, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double const * const imInput, double const * const imInputMagnitude, double const * const imInputGradient, double const * const imInputUnary, double const * const imInputEnergy, double const * const imInputInstance, double * const imCurr, double * const imShift, double * const imVisible, double * const totalDistortion, double * const imEnergyOut);

int sc_seam_remove(int const * const vecSeam, int const unaryinput, int const refinement, double const blend, int const logspeedup, double * const imCurr,double * const imMagnitude, double * const imEnergy, int * const imShift, double * const imInstance, double * const imDistortion, int & currC, int const C, int const R, double * const imUnary, int * const imVisible, int * const imNumber);

int sc_optimize(double * const imEnergy, int const currC, int const C, int const R, int & now_crop, int const un_small, int const infinity, int const unaryinput, int const logspeedup, double const * const imInstance, int * const imBound, int & bound_max, int * const vecSeam);

double sc_dp(double const * const imEnergy, int const currC, int const C, int const R, int * const vecSeam);
double sc_dp_logspeedup(double const * const imEnergy, int const currC, int const C, int const R, int const * const vecBoundL, int const * const vecBoundR, int * const vecSeam);

int sc_pairwiseabstract(int const r, int const c, int const R, int const currC, int const C, double const un_small, absPairwise const * const fncPairwise, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, double * const imEnergy);

int sc_shiftabstract(int const r, int const c, int const R, int const currC, int const C, int const it, double const shiftweight, int const * const imShift, double const * const imInputUnary, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double * const imEnergy);

int sc_energy(int const R, int const currC, int const C, int const it, int const * const vecSeam, int const startrange, int const endrange, double const un_small, double const infinity, int const unary, int const unaryinput, int const total, double const * const imDistortion, double const * const imUnary, double const * const imInstance, absPairwise const * const fncPairwise, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, double const * const imInputUnary, int const * const imShift, int const * const imNumber, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double const shiftweight, double * const imEnergy);

int sc_distortion_remove(int const R, int const currC, int const C, int const * const vecSeam, double * const imEnergy, absPairwise const * const fncPairwise, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double const shiftweight, double const * const imInputUnary, int const it, double * const imDistortion);

// inline methods

inline int index2(int r, int c, int R) {
    return c*R + r;
}

inline int index3(int r, int c, int t, int R, int C) {
    return t*(R*C) + c*R + r;
}

inline int signum(int x) {
  return (x>0) - (x<0);
}

#endif
