/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "sc.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

int sc_distortion_remove(int const R, int const currC, int const C, int const * const vecSeam, double * const imEnergy, absPairwise const * const fncPairwise, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double const shiftweight, double const * const imInputUnary, int const it, double * const imDistortion) {
  
  for(int r=R-1; r>=0; r--) {

    imDistortion[index3(r,vecSeam[r],0,R,C)] = 0.0;
    imDistortion[index3(r,vecSeam[r],1,R,C)] = 0.0;

    // calculate distortion energy
    sc_pairwiseabstract(r,vecSeam[r],R,currC,C,0.0,fncPairwise,pairwiseorder,blend,imCurr,imMagnitude,imInput,imInputMagnitude,imShift,imNumber,imEnergy);
    sc_shiftabstract(r,vecSeam[r],R,currC,C,it,shiftweight,imShift,imInputUnary,fncShift,fncShiftOpV,fncShiftOpH,imEnergy);
    //(this is ok as these terms will be remoted later)

    assert(imEnergy[index3(r,vecSeam[r],0,R,C)]>=0);
    assert(imEnergy[index3(r,vecSeam[r],1,R,C)]>=0);
    assert(imEnergy[index3(r,vecSeam[r],2,R,C)]>=0);

    if(((vecSeam[r]-1)>=0)&&((vecSeam[r]-1)<currC)) {
      imDistortion[index3(r,vecSeam[r]-1,0,R,C)] = imEnergy[index3(r,vecSeam[r],1,R,C)];
    }
    for(int c=vecSeam[r]-1; c<=vecSeam[r]+1; c=c+2) {
      if((c>=0)&&(c<currC)&&(r>0)) {
	if(vecSeam[r-1]==c) {
	  if(c==(vecSeam[r]-1)) {
	    imDistortion[index3(r,c,1,R,C)] = imEnergy[index3(r,vecSeam[r],0,R,C)];
	  } else {
	    imDistortion[index3(r,c,1,R,C)] = imEnergy[index3(r,vecSeam[r],2,R,C)];
	  }
	}
      }
    }
  }
  
  return 1;

}
