
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 


#include "sc.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

double sc_dp(double const * const imEnergy, int const currC, int const C, int const R, int * const vecSeam) {

  double * imCumEnergy;
  int * imLabel;
  imCumEnergy = (double*)malloc(R*currC*sizeof(double));
  imLabel = (int*)malloc(R*currC*sizeof(int));

  //forward pass
            
  for(int c=0;c<currC;c++) {                
    imCumEnergy[index2(0,c,R)] = imEnergy[index3(0,c,1,R,C)];
    imLabel[index2(0,c,R)] = 0;
  }
            
  for(int r=1;r<R;r++) {
    for(int c=0;c<currC;c++) { 
      double bestCumEnergy = 0;
      int bestLabel = -1;
                    
      for(int lastc=-1;lastc<2;lastc++) {
		      
	if(lastc==-1)
	  lastc = 0;
	else if(lastc==0)
	  lastc = -1;

	if(((lastc+c)>=0)&&((lastc+c)<currC)) {
	  int thisLabel = lastc + 2;
	  double thisCumEnergy = imCumEnergy[index2(r-1,lastc+c,R)] + imEnergy[index3(r,c,thisLabel-1,R,C)];

	  if(thisLabel!=2) {;
	    thisCumEnergy += imEnergy[index3(r,c,1,R,C)];
	  }

	  if((bestLabel==-1)||(thisCumEnergy < bestCumEnergy)) {
	    bestLabel = thisLabel;
	    bestCumEnergy = thisCumEnergy;
	  }
	}

	if(lastc==-1)
	  lastc = 0;
	else if(lastc==0)
	  lastc = -1;
      }

      assert(bestLabel>=0);
                    
      imCumEnergy[index2(r,c,R)] = bestCumEnergy;
      imLabel[index2(r,c,R)] = bestLabel;
    }
  }
        
  //backward pass
  double bestCumEnergy = imCumEnergy[index2(R-1,0,R)];
  int bestC = 0;
  for(int c=1;c<currC;c++) {
    if(imCumEnergy[index2(R-1,c,R)] < bestCumEnergy) {
      bestCumEnergy = imCumEnergy[index2(R-1,c,R)];
      bestC = c;
    }
  }
  assert((bestC>=0)&&(bestC<currC));
  vecSeam[R-1] = bestC;
  for(int r=(R-1);r>0;r--) {
    vecSeam[r-1] = vecSeam[r] + imLabel[index2(r,vecSeam[r],R)] - 2;
    assert((vecSeam[r-1]>=0)&&(vecSeam[r-1]<currC));
  }
  
  free(imCumEnergy);
  free(imLabel);
  
  return bestCumEnergy;

}
