
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 


#include "sc.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

int sc_energy(int const R, int const currC, int const C, int const it, int const * const vecSeam, int const startrange, int const endrange, double const un_small, double const infinity, int const unary, int const unaryinput, int const total, double const * const imDistortion, double const * const imUnary, double const * const imInstance, absPairwise const * const fncPairwise, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, double const * const imInputUnary, int const * const imShift, int const * const imNumber, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double const shiftweight, double * const imEnergy) {

  for(int r=0; r<R; r++) {
    for(int c=vecSeam[r]-startrange; c<=vecSeam[r]+endrange; c++) {
      if((c>=0)&&(c<currC)) {
      
	//pairwise term
	sc_pairwiseabstract(r,c,R,currC,C,un_small,fncPairwise,pairwiseorder,blend,imCurr,imMagnitude,imInput,imInputMagnitude,imShift,imNumber,imEnergy);

	//shift term
	sc_shiftabstract(r,c,R,currC,C,it,shiftweight,imShift,imInputUnary,fncShift,fncShiftOpV,fncShiftOpH,imEnergy);

	assert(imEnergy[index3(r,c,0,R,C)]>=0);
	assert(imEnergy[index3(r,c,1,R,C)]>=0);
	assert(imEnergy[index3(r,c,2,R,C)]>=0);
      
	//remove cost of previous distortion
	if(total==1) {
	  imEnergy[index3(r,c,1,R,C)] -= (imDistortion[index3(r,c,0,R,C)] + imDistortion[index3(r,c,1,R,C)]);
	  if(c>0) {
	    imEnergy[index3(r,c,0,R,C)] -= imDistortion[index3(r,c-1,1,R,C)];
	    imEnergy[index3(r,c,1,R,C)] -= imDistortion[index3(r,c-1,0,R,C)];
	  }
	  if(c<(currC-1))
	    imEnergy[index3(r,c,2,R,C)] -= imDistortion[index3(r,c+1,1,R,C)];
	}
      
	//add unary terms
	if(unary>0) {
	  imEnergy[index3(r,c,1,R,C)] += imUnary[index2(r,c,R)];
	}

	//if using input annotation
	if(unaryinput==1) {
	  if(imInstance[index2(r,c,R)]>0)
	    imEnergy[index3(r,c,1,R,C)] += infinity;
	}
      }
    }
  }
 
  return 1;

}
