
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 


#include "sc.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

int sc_optimize(double * const imEnergy, int const currC, int const C, int const R, int & now_crop, int const un_small, int const infinity, int const unaryinput, int const logspeedup, double const * const imInstance, int * const imBound, int & bound_max, int * const vecSeam) {

  double seamcost = 0.0;

  int * vecBoundL, * vecBoundR;
  int bound = 0;
  vecBoundL = (int*)malloc(R*sizeof(int));
  vecBoundR = (int*)malloc(R*sizeof(int));

  if(logspeedup>0) {
    bound = rand() % bound_max;
 
    for(int r=0; r<R; r++) {
      vecBoundL[r] = imBound[index2(r,bound,R)];
      vecBoundR[r] = imBound[index2(r,bound+1,R)];
    }

    seamcost = sc_dp_logspeedup(imEnergy,currC,C,R,vecBoundL,vecBoundR,vecSeam); 
  
  } else {

    seamcost = sc_dp(imEnergy,currC,C,R,vecSeam);

  }

  int recurse = 0;
  if((unaryinput==1)&&(now_crop==0)&&(seamcost>infinity)) {
    now_crop = 1;
    for(int r=0; r<R; r++) {
      if(imInstance[index2(r,0,R)]>0)
	imEnergy[index3(r,0,1,R,C)] = un_small;
      if(imInstance[index2(r,currC-1,R)]>0)
	imEnergy[index3(r,currC-1,1,R,C)] = un_small;
    }
    recurse = 1;
    sc_optimize(imEnergy,currC,C,R,now_crop,un_small,infinity,unaryinput,logspeedup,imInstance,imBound,bound_max,vecSeam);
  }

  if((logspeedup>0)&&(recurse==0)) {
    int addthis = 1;
    int boundtouch = 0;
    for(int r=0; r<R; r++) {
      if((vecSeam[r]==vecBoundL[r])||(vecSeam[r]==(vecBoundR[r]-1)))
	addthis = 0;
      if(vecBoundL[r]==(vecBoundR[r]-1))
	boundtouch = 1;
    }

    for(int r=0; r<R; r++) {
      for(int c=bound+1; c<=bound_max; c++)
	imBound[index2(r,c,R)]--;
    }

    if(addthis==1) {
      bound_max++;
      for(int r=0; r<R; r++) {
	for(int c=bound_max; c>=bound+2; c--) {
	  imBound[index2(r,c,R)] = imBound[index2(r,c-1,R)];
	}
	imBound[index2(r,bound+1,R)] = vecSeam[r];
      }
    } else if(boundtouch==1) {
      if(bound==(currC-2)) {
	for(int r=0; r<R; r++) {
	  for(int c=bound; c<bound_max ; c++) {
	    imBound[index2(r,c,R)] = imBound[index2(r,c+1,R)];
	  }
	}
      } else {
	for(int r=0; r<R; r++) {
	  for(int c=bound+1; c<bound_max ; c++) {
	    imBound[index2(r,c,R)] = imBound[index2(r,c+1,R)];
	  }
	}
      }
      bound_max--;
      }
    }
  
  free(vecBoundL);
  free(vecBoundR); 
  
  return 1;

}
