
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "sc.h"
#include <string.h>
#include <math.h>

double pairwise1(int const R, int const currC, int const C, int const domain, int const term, double const pairwiseorder, double const blend, double const * const imCurr, double const * const imMagnitude, double const * const imInput, double const * const imInputMagnitude, int const * const imShift, int const * const imNumber, int const * const r, int const * const c) {

  if(domain==0) {
    // Curr
    if(term==1) {
      return pow( fabs( imCurr[index3(r[0],c[0],0,R,C)] - imCurr[index3(r[1],c[1],0,R,C)]),pairwiseorder ) +
	     pow( fabs( imCurr[index3(r[0],c[0],1,R,C)] - imCurr[index3(r[1],c[1],1,R,C)]),pairwiseorder ) +
	     pow( fabs( imCurr[index3(r[0],c[0],2,R,C)] - imCurr[index3(r[1],c[1],2,R,C)]),pairwiseorder );
    } else {
      return pow( fabs( (1-blend)*(imCurr[index3(r[0],c[0],0,R,C)] - imCurr[index3(r[1],c[1],0,R,C)]) +
			   blend*(imCurr[index3(r[2],c[2],0,R,C)] - imCurr[index3(r[3],c[3],0,R,C)])),pairwiseorder ) +
	     pow( fabs( (1-blend)*(imCurr[index3(r[0],c[0],1,R,C)] - imCurr[index3(r[1],c[1],1,R,C)]) + 
			   blend*(imCurr[index3(r[2],c[2],1,R,C)] - imCurr[index3(r[3],c[3],1,R,C)])),pairwiseorder ) +
             pow( fabs( (1-blend)*(imCurr[index3(r[0],c[0],2,R,C)] - imCurr[index3(r[1],c[1],2,R,C)]) +
			   blend*(imCurr[index3(r[2],c[2],2,R,C)] - imCurr[index3(r[3],c[3],2,R,C)])),pairwiseorder );
    }
  } else {
    // Input
    if(term==1) {
      return pow( fabs( imInput[index3(r[0],c[0],0,R,C)] - imInput[index3(r[1],c[1],0,R,C)]),pairwiseorder ) +
	     pow( fabs( imInput[index3(r[0],c[0],1,R,C)] - imInput[index3(r[1],c[1],1,R,C)]),pairwiseorder ) +
             pow( fabs( imInput[index3(r[0],c[0],2,R,C)] - imInput[index3(r[1],c[1],2,R,C)]),pairwiseorder );
    } else if(term==2) {
      return pow( fabs( imCurr[index3(r[0],imNumber[index2(r[0],c[0],R)],0,R,currC)] - imInput[index3(r[1],c[1],0,R,C)]),pairwiseorder ) +
             pow( fabs( imCurr[index3(r[0],imNumber[index2(r[0],c[0],R)],1,R,C)] - imInput[index3(r[1],c[1],1,R,C)]),pairwiseorder ) +
             pow( fabs( imCurr[index3(r[0],imNumber[index2(r[0],c[0],R)],2,R,C)] - imInput[index3(r[1],c[1],2,R,C)]),pairwiseorder );
    } else if(term==3) {
      return pow( fabs( imInput[index3(r[0],c[0],0,R,C)] - imCurr[index3(r[1],imNumber[index2(r[1],c[1],R)],0,R,C)]),pairwiseorder ) +
             pow( fabs( imInput[index3(r[0],c[0],1,R,C)] - imCurr[index3(r[1],imNumber[index2(r[1],c[1],R)],1,R,C)]),pairwiseorder ) +
             pow( fabs( imInput[index3(r[0],c[0],2,R,C)] - imCurr[index3(r[1],imNumber[index2(r[1],c[1],R)],2,R,C)]),pairwiseorder );
    } else {
      return pow( fabs( imCurr[index3(r[0],imNumber[index2(r[0],c[0],R)],0,R,C)] - imCurr[index3(r[1],imNumber[index2(r[1],c[1],R)],0,R,C)]),pairwiseorder ) +
	     pow( fabs( imCurr[index3(r[0],imNumber[index2(r[0],c[0],R)],1,R,C)] - imCurr[index3(r[1],imNumber[index2(r[1],c[1],R)],1,R,C)]),pairwiseorder ) +
	     pow( fabs( imCurr[index3(r[0],imNumber[index2(r[0],c[0],R)],2,R,C)] - imCurr[index3(r[1],imNumber[index2(r[1],c[1],R)],2,R,C)]),pairwiseorder );
    }
  }

} 
