
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "sc.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

int sc_seam_remove(int const * const vecSeam, int const unaryinput, int const refinement, double const blend, int const logspeedup, double * const imCurr,double * const imMagnitude, double * const imEnergy, int * const imShift, double * const imInstance, double * const imDistortion, int & currC, int const C, int const R, double * const imUnary, int * const imVisible, int * const imNumber) {

  // note: move pixels leftwards wherever there is a seam, but do not reallocate memory
  for(int r=0; r<R; r++) {
    assert((vecSeam[r]>=0)&&(vecSeam[r]<currC));
    assert(((vecSeam[r]+imShift[index2(r,vecSeam[r],R)])>=0)&&((vecSeam[r]+imShift[index2(r,vecSeam[r],R)])<C));
    
    imVisible[index2(r,vecSeam[r]+imShift[index2(r,vecSeam[r],R)],R)] = 0;
    for(int c=vecSeam[r]+imShift[index2(r,vecSeam[r],R)]; c<C; c++) {
      imNumber[index2(r,c,R)]--;
    }

    if(blend>0.0) {
      if(vecSeam[r]>0) {
	imCurr[index3(r,vecSeam[r]-1,0,R,C)] = (1.0-blend)*imCurr[index3(r,vecSeam[r]-1,0,R,C)] + blend*imCurr[index3(r,vecSeam[r],0,R,C)];
	imCurr[index3(r,vecSeam[r]-1,1,R,C)] = (1.0-blend)*imCurr[index3(r,vecSeam[r]-1,1,R,C)] + blend*imCurr[index3(r,vecSeam[r],1,R,C)];
	imCurr[index3(r,vecSeam[r]-1,2,R,C)] = (1.0-blend)*imCurr[index3(r,vecSeam[r]-1,2,R,C)] + blend*imCurr[index3(r,vecSeam[r],2,R,C)];
	imMagnitude[index2(r,vecSeam[r]-1,R)] = sqrt(pow(imCurr[index3(r,vecSeam[r]-1,0,R,C)],2.0) + 
						     pow(imCurr[index3(r,vecSeam[r]-1,1,R,C)],2.0) + 
						     pow(imCurr[index3(r,vecSeam[r]-1,2,R,C)],2.0)) / sqrt(3);
      }
      if(vecSeam[r]<(currC-1)) {
	imCurr[index3(r,vecSeam[r]+1,0,R,C)] = (1.0-blend)*imCurr[index3(r,vecSeam[r]+1,0,R,C)] + blend*imCurr[index3(r,vecSeam[r],0,R,C)];
	imCurr[index3(r,vecSeam[r]+1,1,R,C)] = (1.0-blend)*imCurr[index3(r,vecSeam[r]+1,1,R,C)] + blend*imCurr[index3(r,vecSeam[r],1,R,C)];
	imCurr[index3(r,vecSeam[r]+1,2,R,C)] = (1.0-blend)*imCurr[index3(r,vecSeam[r]+1,2,R,C)] + blend*imCurr[index3(r,vecSeam[r],2,R,C)];
	imMagnitude[index2(r,vecSeam[r]+1,R)] = sqrt(pow(imCurr[index3(r,vecSeam[r]+1,0,R,C)],2.0) + 
						     pow(imCurr[index3(r,vecSeam[r]+1,1,R,C)],2.0) + 
						     pow(imCurr[index3(r,vecSeam[r]+1,2,R,C)],2.0)) / sqrt(3);
      }
    }

    for(int c=vecSeam[r]; c<(currC-1); c++) {
      imCurr[index3(r,c,0,R,C)] = imCurr[index3(r,c+1,0,R,C)];
      imCurr[index3(r,c,1,R,C)] = imCurr[index3(r,c+1,1,R,C)];
      imCurr[index3(r,c,2,R,C)] = imCurr[index3(r,c+1,2,R,C)];
      imMagnitude[index2(r,c,R)] = imMagnitude[index2(r,c+1,R)];
      imUnary[index2(r,c,R)] = imUnary[index2(r,c+1,R)];
      imDistortion[index3(r,c,0,R,C)] = imDistortion[index3(r,c+1,0,R,C)];
      imDistortion[index3(r,c,1,R,C)] = imDistortion[index3(r,c+1,1,R,C)];
      imEnergy[index3(r,c,0,R,C)] = imEnergy[index3(r,c+1,0,R,C)];
      imEnergy[index3(r,c,1,R,C)] = imEnergy[index3(r,c+1,1,R,C)];
      imEnergy[index3(r,c,2,R,C)] = imEnergy[index3(r,c+1,2,R,C)];
      imInstance[index2(r,c,R)] = imInstance[index2(r,c+1,R)];
      imShift[index2(r,c,R)] = imShift[index2(r,c+1,R)] + 1;
    }
    imDistortion[index3(r,currC-1,0,R,C)] = 0.0;
    imDistortion[index3(r,currC-1,1,R,C)] = 0.0;
    }

  currC = currC - 1;
  
  return 1;

}
