
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "sc.h"
#include <string.h>
#include <math.h>

double shift4(int const R, int const C, int const it, int const domain, int const term, int const shiftweight, int const * const imShift, double const * const imInputUnary, int const * const r, int const * const c) {

  int c_min = 0, c_max = 0;
  if(domain==0) {
    // Curr
    if(term==1) {
      c_min = c[0] + imShift[index2(r[0],c[0],R)] + 1;
      c_max = c[1] + imShift[index2(r[1],c[1],R)] - 1;
      if(c_max < c_min) {
	int c_tmp = c_min;
	c_min = c_max;
	c_max = c_tmp;
      }
    } else if(term==2) {
      c_min = 0;
      c_max = c[0] + imShift[index2(r[0],c[0],R)] - 1;
      if(c_max < c_min) {
	int c_tmp = c_min;
	c_min = c_max;
	c_max = c_tmp;
      }
    } else if(term==3) {
      c_min = c[0] + imShift[index2(r[0],c[0],R)] + 1;
      c_max = C-1;
      if(c_max < c_min) {
	int c_tmp = c_min;
	c_min = c_max;
	c_max = c_tmp;
      }
    }
  } else {
    // Input
    if(term==1) {
      c_min = c[0];
      c_max = c[1];
      if(c_max < c_min) {
	int c_tmp = c_min;
	c_min = c_max;
	c_max = c_tmp;
      }
    } else if(term==2) {
      c_min = 0;
      c_max = c[0];
      if(c_max < c_min) {
	int c_tmp = c_min;
	c_min = c_max;
	c_max = c_tmp;
      }
    } else if(term==3) {
      c_min = c[0];
      c_max = C-1;
      if(c_max < c_min) {
	int c_tmp = c_min;
	c_min = c_max;
	c_max = c_tmp;
      }
    }
  }

  double sum = 0.0;
  for(int thisc = c_min; thisc <= c_max; thisc++) {
    sum += imInputUnary[index2(r[0],thisc,R)];
  }

  return shiftweight * (((double)c_max-double(c_min)) / ((double)c_max - (double)c_min + 1.0)) * sum;

} 
