% ***Seam carving with dynamic programming***
% Alex Mansfield, ETH Zurich, Jan 2010

%    ---------------------------------------------------------------------
%
%    MATLAB/MEX Generalized seam carving      
%    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    ---------------------------------------------------------------------

function [imCurr imShift imVisible] = seamcarve_mex(filepreImage,filepostImage,nameImage,fileTime,infinity,un_small,S,pairwise,pairwiseorder,unary,unaryorder,unaryweight,unaryinput,shiftterm,shiftweight,refinement,boolFigures,boolFiles,blend,total,logspeedup,fncPairwise,fncShift,fncShiftOpV,fncShiftOpH)

ts = tic;
    
    % Initialization ------------------------------------------------------

    scrsz = get(0,'ScreenSize');

    imInput = double(imread([filepreImage filepostImage]));
    [R C colours] = size(imInput);
    currC = C;

    if(round(S)~=S)
        S = round(S*C); 
    end

    imMagnitude = double(zeros(R,C));
    imEnergy = double(zeros(R,C,3));
    imGradient = double(zeros(R,C,2));
    imUnary = double(zeros(R,C));
    imInstance = double(zeros(R,C));
    imShift = double(zeros(R,C));
    imNumber = repmat(1:C,R,1);    

    if(exist([filepreImage '/' nameImage '_pairwise_' int2str(pairwise) '_o' num2str(pairwiseorder) '_b' num2str(blend) '.mat']))
        load([filepreImage '/' nameImage '_pairwise_' int2str(pairwise) '_o' num2str(pairwiseorder) '_b' num2str(blend) '.mat']);
    else 
        imMagnitude = sqrt((double(imInput(:,:,1)).^2+double(imInput(:,:,2)).^2+double(imInput(:,:,3)).^2)) / sqrt(3);
        
        for r = 1:R
            for c = 1:C
                imEnergy = pairwiseAbstract(r,c,un_small,imEnergy,R,currC,fncPairwise,pairwiseorder,blend,imInput,imMagnitude,imInput,imMagnitude,imShift,imNumber);
            end
        end
        
        save([filepreImage '/' nameImage '_pairwise_' num2str(pairwise) '_o' num2str(pairwiseorder) '_b' num2str(blend) '.mat'],'imMagnitude','imEnergy');
    end

    if(exist([filepreImage '/' nameImage '_gradient.mat']))
        load([filepreImage '/' nameImage '_gradient.mat']);
    else 
        imGradient(:,:,1) = nlfilter(imMagnitude,[3 3],@sobelgradientL1);
        imGradient(:,:,2) = nlfilter(imMagnitude',[3 3],@sobelgradientL1)';
        save([filepreImage '/' nameImage '_gradient.mat'],'imGradient');
    end 

    if(exist([filepreImage '/' nameImage '_unary.mat']))
        load([filepreImage '/' nameImage '_unary.mat']);
    else 
        imUnary = imGradient(:,:,1) + imGradient(:,:,2);
        save([filepreImage '/' nameImage '_unary.mat'],'imUnary');
    end 
    imUnary = imUnary.^unaryorder;
    imUnary = imUnary * unaryweight;

    if(unaryinput==1)
        imInstance = imread([filepreImage '/' nameImage '_instance.bmp']);
        imInstance = double(imInstance(:,:,1));
    end  
    
    if(unary>0)
        imEnergy(:,:,2) = imEnergy(:,:,2) + imUnary;
    end

    if(unaryinput==1)
        % infinite costs for objects
        index = imInstance(:,:)>0;
        temp = zeros(size(imEnergy(:,:,1),1),size(imEnergy(:,:,1),2));
        temp(index) = infinity;
        imEnergy(:,:,2) = imEnergy(:,:,2) + temp;
    end
    
    % Go to mex -----------------------------------------------------------
    
    [imCurr imShift imVisible totalDistortion imEnergyOut] = seamcarve_inside_mex(pairwise,pairwiseorder,unary,unaryorder,unaryweight,unaryinput,shiftterm,shiftweight,refinement,blend,total,logspeedup,infinity,un_small,abs(S),func2str(fncPairwise),func2str(fncShift),func2str(fncShiftOpV),func2str(fncShiftOpH),imInput,imMagnitude,imGradient,imUnary,imEnergy,imInstance);
    
    imCurr = uint8(imCurr);
    imShift = imShift/abs(S)*255;
    imVisible = uint8(imVisible*255);
    
    totalDistortion
    
    if(boolFigures)
        figure(1);
        sc_figures(S,R,imInput,imCurr,imShift,imVisible,imEnergyOut);
        
        imwrite(uint8(imCurr),'out.bmp');
        
    end
    if(boolFiles)

        % Outputs -------------------------------------------------------------

        if(logspeedup>0)
            imwrite(uint8(imCurr),[filepreImage '/' nameImage '_p' int2str(pairwise) '_po' num2str(pairwiseorder) '_u' int2str(unary) '_uo' num2str(unaryorder) '_uw' num2str(unaryweight) '_ui' int2str(unaryinput) '_st' int2str(shiftterm) '_sw' num2str(shiftweight) '_r' int2str(refinement) '_b' num2str(blend) '_t' int2str(total) '_l' int2str(logspeedup) '_' int2str(S) '.bmp']);
            imwrite(uint8(imShift/abs(S)*255),jet(256),[filepreImage '/' nameImage '_shift' '_p' int2str(pairwise) '_po' num2str(pairwiseorder) '_u' int2str(unary) '_uo' num2str(unaryorder) '_uw' num2str(unaryweight) '_ui' int2str(unaryinput) '_st' int2str(shiftterm) '_sw' num2str(shiftweight) '_r' int2str(refinement) '_b' num2str(blend) '_t' int2str(total) '_l' int2str(logspeedup) '_' int2str(S) '.bmp']);
            imwrite(uint8(imVisible*255),[filepreImage '/' nameImage '_visible' '_p' int2str(pairwise) '_po' num2str(pairwiseorder) '_u' int2str(unary) '_uo' num2str(unaryorder) '_uw' num2str(unaryweight) '_ui' int2str(unaryinput) '_st' int2str(shiftterm) '_sw' num2str(shiftweight) '_r' int2str(refinement) '_b' num2str(blend) '_t' int2str(total) '_l' int2str(logspeedup) '_' int2str(S) '.bmp']);
        else
            imwrite(uint8(imCurr),[filepreImage '/' nameImage '_p' int2str(pairwise) '_po' num2str(pairwiseorder) '_u' int2str(unary) '_uo' num2str(unaryorder) '_uw' num2str(unaryweight) '_ui' int2str(unaryinput) '_st' int2str(shiftterm) '_sw' num2str(shiftweight) '_r' int2str(refinement) '_b' num2str(blend) '_t' int2str(total) '_' int2str(S) '.bmp']);
            imwrite(uint8(imShift/abs(S)*255),jet(256),[filepreImage '/' nameImage '_shift' '_p' int2str(pairwise) '_po' num2str(pairwiseorder) '_u' int2str(unary) '_uo' num2str(unaryorder) '_uw' num2str(unaryweight) '_ui' int2str(unaryinput) '_st' int2str(shiftterm) '_sw' num2str(shiftweight) '_r' int2str(refinement) '_b' num2str(blend) '_t' int2str(total) '_' int2str(S) '.bmp']);
            imwrite(uint8(imVisible*255),[filepreImage '/' nameImage '_visible' '_p' int2str(pairwise) '_po' num2str(pairwiseorder) '_u' int2str(unary) '_uo' num2str(unaryorder) '_uw' num2str(unaryweight) '_ui' int2str(unaryinput) '_st' int2str(shiftterm) '_sw' num2str(shiftweight) '_r' int2str(refinement) '_b' num2str(blend) '_t' int2str(total) '_' int2str(S) '.bmp']);
        end
        
        t = toc(ts);
        fTime = fopen(fileTime,'a');
        fprintf(fTime,'SC,%s,%i,%i,%f,%i,%f,%i,%i,%f,%f,%i,%f,%i,%i,%f,%f\n',[filepreImage filepostImage],S,pairwise,pairwiseorder,unary,unaryorder,unaryweight,unaryinput,shiftterm,shiftweight,refinement,blend,total,logspeedup,totalDistortion,t);
        fclose(fTime);
    end

end

function sc_figures(S,R,imInput,imCurr,imShift,imVisible,imEnergy)

set(1,'Name',['Iteration ' int2str(S)]);
subplot(2,3,1); 
image(uint8(imCurr));
title('Image');
subplot(2,3,2); 
image(uint8(imShift/abs(S)*255));
colormap jet(256);
title('Shift');
subplot(2,3,4); 
image(uint8(imInput));
title('Input');
subplot(2,3,5); 
image(imVisible*255);
title('Visible');
subplot(2,3,6); 
image(imEnergy(:,:,2));
title('Energy');

drawnow;

end